package net.wanji.com.netty.handler;

import cn.hutool.core.util.HexUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import lombok.extern.slf4j.Slf4j;
import net.wanji.com.cache.CrossInfoCache;
import net.wanji.com.cache.netty.NettyMessageCache;
import net.wanji.com.common.enums.CommandResultSign;
import net.wanji.com.netty.agreementcommon.AgreementCommon;
import net.wanji.com.netty.pojo.CommandPojo;
import net.wanji.com.netty.response.CommandResponseService;
import net.wanji.com.pojo.netty.MessageResultPojo;
import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.databus.po.CrossInfoPO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.util.Objects;

@Slf4j
public class NettyServerHandler extends SimpleChannelInboundHandler<DatagramPacket> {

    @Resource
    private CrossInfoCache crossInfoCache;
    private ThreadPoolTaskExecutor threadPoolExecutor;

    public NettyServerHandler(ThreadPoolTaskExecutor threadPoolExecutor) {
        this.threadPoolExecutor = threadPoolExecutor;
    }

    /**
     * 读取消息
     *
     * @param ctx
     * @param packet
     * @throws Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket packet) throws Exception {
        log.info("收到服务端包数据： {}", packet);
        ByteBuf content = packet.content();
        InetSocketAddress sender = packet.sender();
        log.info("收到消息地址： {}, 端口： {}", sender.getAddress(), sender.getPort());
        int i = content.readableBytes();
        byte[] b = new byte[i];
        content.readBytes(b);
        String s = HexUtil.encodeHexStr(b);
        log.info("收到返回消息：{}", s);
//        String hexString = HexUtils.bytesToHexString(b);


        String key = ctx.channel().remoteAddress().toString();
//        String data = msg.toString();
        String data = null;
        log.debug("receive server data :{}", data);

        String hexSign = AgreementCommon.getHexSign(data);
        if (StringUtils.isEmpty(hexSign)) {
            log.error("data unique hex sign error");
            return;
        }
        String className = CommandResultSign.getClassNameByHexSign(hexSign);
        CommandResponseService commandResponseService = (CommandResponseService) ServiceBeanContext.getBean(className);
        CommandPojo commandPojo = getCommandPojo(key, data);
        String hexResult = commandResponseService.getCommandResponse(commandPojo);
        if (Objects.nonNull(hexResult)) {
            setResultData(key, hexResult);
        }
    }

    private void setResultData(String key, String hexResult) {
        MessageResultPojo resultPojo = NettyMessageCache.NETTY_MESSAGE_RESULT_MAP.get(key);
        if (resultPojo != null) {
            resultPojo.setHexMessageResult(hexResult);
            resultPojo.getCountDownLatch().countDown();
        }
    }

    private CommandPojo getCommandPojo(String key, String data) {
        CrossInfoPO crossInfo = crossInfoCache.getCrossInfo(key);
        CommandPojo commandPojo = new CommandPojo();
        commandPojo.setCrossId(crossInfo.getId());
        commandPojo.setSignalId(crossInfo.getCode());
        commandPojo.setSignalIp(crossInfo.getIp());
        commandPojo.setPort(crossInfo.getPort());
        commandPojo.setResponseMsg(data);
        return commandPojo;
    }
}
